#ifndef DUNE_SUBGRID_ENTITY_POINTER_HH
#define DUNE_SUBGRID_ENTITY_POINTER_HH

/** \file
* \brief The SubGridEntityPointer class
*/

namespace Dune {


/** Acts as a pointer to an  entities of a given codimension.
*/
template<int codim, class GridImp>
class SubGridEntityPointer /*:
                             public EntityPointerDefaultImplementation <codim, GridImp, Dune::SubGridEntityPointer<codim,GridImp> > */
{
    
        enum { dim = GridImp::dimension };

    
    public:
    //! export the type of the EntityPointer Implementation.
    //! Necessary for the typeconversion between Iterators and EntityPointer
    typedef SubGridEntityPointer EntityPointerImp;

    /** \brief Codimension of entity pointed to */
    enum { codimension = codim };

        typedef typename GridImp::template Codim<codim>::Entity Entity;
        
        typedef SubGridEntityPointer<codim,GridImp> Base;
    
        // The codimension of this entitypointer wrt the host grid
        enum {CodimInHostGrid = GridImp::HostGridType::dimension - GridImp::dimension + codim};
    
        // EntityPointer to the equivalent entity in the host grid
        typedef typename GridImp::HostGridType::Traits::template Codim<CodimInHostGrid>::EntityPointer HostGridEntityPointer;
        
        
        //! constructor
        SubGridEntityPointer (const GridImp* subGrid, const HostGridEntityPointer& hostEntity_) :
            subGrid_(subGrid),
            virtualEntity_(subGrid, hostEntity_)
        {
        }
    
    /** \brief Constructor from entity */
    SubGridEntityPointer(const SubGridEntity<codim,dim,GridImp>& entity) 
        : subGrid_(entity.subGrid_),
          virtualEntity_(entity.subGrid_, entity.hostEntity_)
    {
    }
    
        //! equality
        bool equals(const SubGridEntityPointer<codim,GridImp>& i) const {
            return virtualEntity_.getTarget() == i.virtualEntity_.getTarget();
        }
    
        
        //! dereferencing
        Entity& dereference() const {
            return virtualEntity_;
        }

    /** \brief Make the memory footprint of this pointer as small as possible
        \todo Doesn't do anything, currently
    */
    void compactify() {}
        
        //! ask for level of entity
        int level () const {
            return virtualEntity_.level();
        }
    
        
    protected:
    
        const GridImp* subGrid_;
        
        //! virtual entity
        mutable SubGridMakeableEntity<codim,dim,GridImp> virtualEntity_;
    
        
};


} // end namespace Dune

#endif
