#ifndef DUNE_SUBGRIDHIERITERATOR_HH
#define DUNE_SUBGRIDHIERITERATOR_HH

/** \file
* \brief The SubGridHierarchicIterator class
*/

namespace Dune {


//**********************************************************************
//
/** \brief Iterator over the descendants of an entity.
* \ingroup SubGrid
Mesh entities of codimension 0 ("elements") allow to visit all entities of
codimension 0 obtained through nested, hierarchic refinement of the entity.
Iteration over this set of entities is provided by the HierarchicIterator,
starting from a given entity.
*/
template<class GridImp>
class SubGridHierarchicIterator :
        public Dune::SubGridEntityPointer <0,GridImp>
{
    public:
        
        typedef typename GridImp::template Codim<0>::Entity Entity;
    
        typedef SubGridEntity <0, GridImp::dimension, GridImp> SubGridElement;
        
    
        //! the default Constructor
        explicit SubGridHierarchicIterator(const GridImp* subGrid, const SubGridElement& startEntity, int maxLevel) :
            SubGridEntityPointer<0,GridImp>(subGrid, startEntity.hostEntity_->hbegin(maxLevel)),
            subGrid_(subGrid),
            hostGridHierarchicIterator_(startEntity.hostEntity_->hbegin(maxLevel)),
            hostGridHierarchicEndIterator_(startEntity.hostEntity_->hend(maxLevel))
        {
            this->virtualEntity_.setToTarget(hostGridHierarchicIterator_);
            if (hostGridHierarchicIterator_ == hostGridHierarchicEndIterator_)
                return;
            
            if (!subGrid_->template contains<0>(*hostGridHierarchicIterator_))
                increment();
            return;
        }
        
        
        //! \todo Please doc me !
        explicit SubGridHierarchicIterator(const GridImp* subGrid, const SubGridElement& startEntity, int maxLevel, bool endDummy) :
            SubGridEntityPointer<0,GridImp>(subGrid, startEntity.hostEntity_->hend(maxLevel)),
            subGrid_(subGrid),
            hostGridHierarchicIterator_(startEntity.hostEntity_->hbegin(maxLevel)),
            hostGridHierarchicEndIterator_(startEntity.hostEntity_->hend(maxLevel))
        {
            return;
        }
    
        
        //! \todo Please doc me !
        void increment()
        {
            while (true)
            {
                ++hostGridHierarchicIterator_;
                if (hostGridHierarchicIterator_ == hostGridHierarchicEndIterator_)
                    break;
                if (subGrid_->template contains<0>(*hostGridHierarchicIterator_))
                    break;
            }
            
            this->virtualEntity_.setToTarget(hostGridHierarchicIterator_);
        }

        
    private:
    
        const GridImp* subGrid_;

        // Type of the corresponding HierarchicIterator in the host grid
        typedef typename GridImp::HostGridType::template Codim<0>::Entity::HierarchicIterator HostGridHierarchicIterator;
    
        HostGridHierarchicIterator hostGridHierarchicIterator_;
        
        HostGridHierarchicIterator hostGridHierarchicEndIterator_;
};


}  // end namespace Dune

#endif
