#ifndef DUNE_SUBGRIDLEAFITERATOR_HH
#define DUNE_SUBGRIDLEAFITERATOR_HH

/** \file
* \brief The SubGridLeafIterator class
*/

namespace Dune {




/** \brief Iterator over all entities of a given codimension and level of a grid.
*  \ingroup SubGrid
*/
template<int codim, PartitionIteratorType pitype, class GridImp>
class SubGridLeafIterator :
    public Dune::SubGridEntityPointer <codim,GridImp>
{
    private:
    
        enum {dim = GridImp::dimension};
    
        
    public:
    
        
        //! \todo Please doc me !
        explicit SubGridLeafIterator(const GridImp* subGrid) :
            SubGridEntityPointer<codim,GridImp>(subGrid, subGrid->getHostGrid().template lbegin<codim>(0)),
            subGrid_(subGrid),
            hostGridLevelIterator_(subGrid->getHostGrid().template lbegin<codim>(0)),
            hostGridLevelEndIterator_(subGrid->getHostGrid().template lend<codim>(0)),
            level_(0)
        {
            
            // Search for the first leaf entity in the subgrid
            this->virtualEntity_.setToTarget(hostGridLevelIterator_);
            
            if (not(subGrid_->template isLeaf<codim>(*hostGridLevelIterator_)))
                increment();
        }
    
        
        /** \brief Constructor which create the end iterator
        *  \param endDummy Here only to distinguish it from the other constructor
        */
        explicit SubGridLeafIterator(const GridImp* subGrid, bool endDummy) :
            SubGridEntityPointer<codim,GridImp>(subGrid, subGrid->getHostGrid().template lend<codim>(subGrid->maxLevel())),
            subGrid_(subGrid),
            hostGridLevelIterator_(subGrid->getHostGrid().template lbegin<codim>(0)),
            hostGridLevelEndIterator_(subGrid->getHostGrid().template lend<codim>(0))
        {
        }
        
    
        //! prefix increment
        void increment() {
            while (true)
            {
                ++hostGridLevelIterator_;
                if (hostGridLevelIterator_ == hostGridLevelEndIterator_)
                {
                    if (level_ < subGrid_->maxLevel())
                    {
                        ++level_;
                        hostGridLevelIterator_ = subGrid_->getHostGrid().template lbegin<codim>(level_);
                        hostGridLevelEndIterator_ = subGrid_->getHostGrid().template lend<codim>(level_);
                    }
                    else
                        break;
                }
                if (subGrid_->template isLeaf<codim>(*hostGridLevelIterator_))
                    break;
            }
            this->virtualEntity_.setToTarget(hostGridLevelIterator_);
        }
    
    
    private:
    
        // /////////////////////////////////////
        //   Data members
        // /////////////////////////////////////
        const GridImp* subGrid_;
    
        //! \todo Please doc me !
        enum {CodimInHostGrid = GridImp::HostGridType::dimension - GridImp::dimension + codim};
        
        // LevelIterator to the equivalent entity in the host grid
        typedef typename GridImp::HostGridType::Traits::template Codim<CodimInHostGrid>::LevelIterator HostGridLevelIterator;
        typedef typename GridImp::HostGridType::Traits::LevelIndexSet HostGridLevelIndexSet;
        
        //! \todo Please doc me !
        HostGridLevelIterator hostGridLevelIterator_;
        
        //! \todo Please doc me !
        HostGridLevelIterator hostGridLevelEndIterator_;
        
        //! \todo Please doc me !
        int level_;
};


}  // namespace Dune
  
#endif
