#ifndef DUNE_SUBGRIDLEVELITERATOR_HH
#define DUNE_SUBGRIDLEVELITERATOR_HH

/** \file
* \brief The SubGridLevelIterator class and its specializations
*/

namespace Dune {




//**********************************************************************
//
// --SubGridLevelIterator
/** \brief Iterator over all entities of a given codimension and level of a grid.
* \ingroup SubGrid
*/
template<int codim, PartitionIteratorType pitype, class GridImp>
class SubGridLevelIterator :
        public Dune::SubGridEntityPointer <codim,GridImp>
{
    private:
    
        enum {dim = GridImp::dimension};
    
        
    public:
        
        typedef typename GridImp::template Codim<codim>::Entity Entity;
        
        
        //! Constructor
        explicit SubGridLevelIterator(const GridImp* subGrid, int level)
            : SubGridEntityPointer<codim,GridImp>(subGrid, subGrid->getHostGrid().template lbegin<codim>(level)),
              subGrid_(subGrid),
              hostGridLevelIterator_(subGrid->getHostGrid().template lbegin<codim>(level)),
              hostGridLevelEndIterator_(subGrid->getHostGrid().template lend<codim>(level)),
              level_(level)
        {
            // Search for the first entity which is in the subgrid
            this->virtualEntity_.setToTarget(hostGridLevelIterator_);
            if (!subGrid_->template contains<codim>(*hostGridLevelIterator_))
                increment();
        }
        
        
        /** \brief Constructor which create the end iterator
            \param endDummy Here only to distinguish it from the other constructor
        */
        explicit SubGridLevelIterator(const GridImp* subGrid, int level, bool endDummy)
            : 
            SubGridEntityPointer<codim,GridImp>(subGrid, subGrid->getHostGrid().template lend<codim>(level)),
            subGrid_(subGrid),
            hostGridLevelIterator_(subGrid->getHostGrid().template lend<codim>(level)),
            hostGridLevelEndIterator_(subGrid->getHostGrid().template lend<codim>(level)),
            level_(level)
        {
        }
        
        
        //! prefix increment
        void increment() {
            while (true)
            {
                ++hostGridLevelIterator_;
                if (hostGridLevelIterator_ == hostGridLevelEndIterator_)
                    break;
                if (subGrid_->template contains<codim>(*hostGridLevelIterator_))
                    break;
            }
            
            this->virtualEntity_.setToTarget(hostGridLevelIterator_);
        }
        
        
    private:
        
        const GridImp* subGrid_;
    
        // The codimension of this level iterator wrt the host grid
        enum {CodimInHostGrid = GridImp::HostGridType::dimension - GridImp::dimension + codim};
    
        // LevelIterator to the equivalent entity in the host grid
        typedef typename GridImp::HostGridType::Traits::template Codim<CodimInHostGrid>::LevelIterator HostGridLevelIterator;
    
        //! \todo Please doc me !
        HostGridLevelIterator hostGridLevelIterator_;
        
        //! \todo Please doc me !
        HostGridLevelIterator hostGridLevelEndIterator_;
        
        //! \todo Please doc me !
        int level_;

};


}  // namespace Dune
  
#endif
