#include <config.h>

#define DUNE_DEVEL_MODE

#include <dune/grid/onedgrid.hh>
#include <dune/grid/test/gridcheck.cc>
#include <dune/grid/test/checkintersectionit.cc>

#include <dune/subgrid/subgrid.hh>

using namespace Dune;

int main (int argc, char *argv[]) try
{
    typedef OneDGrid GridType;

    typedef GridType::Codim<0>::LevelIterator HostElementIterator;
    typedef SubGrid<1,GridType>::Codim<0>::LevelIterator ElementIterator;

    GridType grid(2,0.0,1.0);

    SubGrid<1,GridType> subGrid(grid);

    // create subgrid
    {
        subGrid.createBegin();
    
        HostElementIterator eIt    = grid.lbegin<0>(0);
        HostElementIterator eEndIt = grid.lend<0>(0);
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.5)
                subGrid.add(eIt);
        }
    
        subGrid.createEnd();
    }
    
    // check subgrid
    std::cout << "Check subgrid<onedgrid>" << std::endl;
    
    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);

    // refine subgrid
    {
        ElementIterator eIt    = subGrid.lbegin<0>(0);
        ElementIterator eEndIt = subGrid.lend<0>(0);
        for (;eIt!=eEndIt; ++eIt) {
            if (eIt->geometry().corner(0)[0] < 0.25)
                subGrid.mark(1, eIt);
        }
        
        subGrid.preAdapt();
        subGrid.adapt();
        subGrid.postAdapt();
    }
    
    // check locally refined subgrid
    std::cout << "Check locally refined subgrid<onedgrid>" << std::endl;

    gridcheck(subGrid);
    checkIntersectionIterator(subGrid);
    return 0;
}

// //////////////////////////////////
//   Error handler
// /////////////////////////////////
catch (Exception e) {
    std::cout << e << std::endl;
    return 1;
}
